pub fn write(output: &mut dyn Write, fmt: Arguments<'_>) -> ResultThis item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).
Takes an output stream and an Arguments struct that can be precompiled with
the format_args! macro.
The arguments will be formatted according to the specified format string into the output stream provided.
Examples
Basic usage:
use std::fmt;
let mut output = String::new();
fmt::write(&mut output, format_args!("Hello {}!", "world"))
.expect("Error occurred while trying to write in String");
assert_eq!(output, "Hello world!");Please note that using write! might be preferable. Example: